/**
 * @brief Este projeto demonstra a amostragem dos sinais de entrada pela tecnica interrupcao
 * @author Wu Shin Ting
 * @date 04/10/2020
 *
 */

#include "derivative.h" /* include peripheral declarations */

#define GPIO_PIN(x)  ((1)<<(x)) ///< macro para gerar uma mascara com 1 no bit x

typedef enum estado_tag {
	DESABILITA,     /**< falso/apaga/desativa/liga */
	HABILITA,      /**< verdadeiro/acende/ativa/fecha */
} estado_type;

typedef struct rgb_bool_tag {
	estado_type vermelho;    
	estado_type verde;        
	estado_type azul;
} rgb_bool_type;

enum cor_tag {
	VERMELHO,
	VERDE,
	AZUL
};

/**
 * @brief inicializa o componente ledRGB
 */
void GPIO_initLedRGB() {
	// Habilita os clocks dos mdulos
	SIM_SCGC5 |= SIM_SCGC5_PORTE_MASK ;    		// para leds

	// Configure pinos do led RGB com estado apagado
	// Funcao GPIO
	PORTE_PCR21 |= PORT_PCR_MUX(0x1);  // vermelho

	PORTE_PCR22 |= PORT_PCR_MUX(0x1);  // verde

	PORTE_PCR23 |= PORT_PCR_MUX(0x1);  // azul
	// Sentido do sinal: saida
	GPIOE_PDDR |= GPIO_PDDR_PDD(GPIO_PIN(21)) |
			GPIO_PDDR_PDD(GPIO_PIN(22)) |
			GPIO_PDDR_PDD(GPIO_PIN(23));
	// Limpar os bits	
	GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(21)) |
			GPIO_PCOR_PTCO(GPIO_PIN(22)) |
			GPIO_PCOR_PTCO(GPIO_PIN(23)) ;	
}

/**
 * @brief seta o estado do componente ledRGB
 * @param[in] estadoR HABILITA/DESABILITA led R
 * @param[in] estadoG HABILITA/DESABILITA led G
 * @param[in] estadoB HABILITA/DESABILITA led B
 */
void GPIO_ledRGB(rgb_bool_type cor) {
	if (cor.vermelho == HABILITA)
		GPIOE_PSOR = GPIO_PSOR_PTSO(GPIO_PIN(21));
	else 
		GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(21));

	if (cor.verde == HABILITA)
		GPIOE_PSOR = GPIO_PSOR_PTSO(GPIO_PIN(22));
	else
		GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(22));

	if (cor.azul == HABILITA)
		GPIOE_PSOR = GPIO_PSOR_PTSO(GPIO_PIN(23));
	else
		GPIOE_PCOR = GPIO_PCOR_PTCO(GPIO_PIN(23));     
}

/**
 * @brief inicializa o componente Botoeiras 
 */
void GPIO_initSwitches () {
	// Configure pinos das botoeiras
	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;				// para botoeiras   					// para leds

	//Funcao GPIO
	PORTA_PCR4 &= ~PORT_PCR_MUX(0x7);			// limpar os bits
	PORTA_PCR4 |= PORT_PCR_MUX(0x1);			// antes de setar 0b001

	PORTA_PCR5 |= PORT_PCR_MUX(0x1);

	PORTA_PCR12 |= PORT_PCR_MUX(0x1);

	//Sentido do sinal: entrada
	GPIOA_PDDR &= ~(GPIO_PDDR_PDD(GPIO_PIN(4)) |
			GPIO_PDDR_PDD(GPIO_PIN(5)) |
			GPIO_PDDR_PDD(GPIO_PIN(12)));    
}

/**
 * @brief Habilita interrupcoes da PORTA no NVIC.
 * @param[in] priority level
 */
void GPIO_enableSwInterrupt(uint8_t priority) {
	/* 
	 * "Non-core interrupt source count" (IRQ = Vector Number (46) - 16, 
	 * Table 3-7 de KL-25 Sub-Family Reference Manual) eh habilitado
	 * ao escrever 1 no bit correpondente do registrador NVIC_ISER 
	 * (B3.4.3 de ARMv6-M Architecture Reference Manual) e a solicitacao de 
	 * interrupcao eh limpa ao escrever 1 no bit correspondente do 
	 * registrador NVIC_ICPR (B3.4.6 de ARMv6-M Architecture Reference Manual) 
	 */
	/* PORTA: Vetor 46, IRQ30, prioridade em NVICIPR7[23:16]
	 * Secao 3.3.2.3 em 
	 * <a href="ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf" target=_manual>ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf</a>
	 */
	NVIC_ISER |= 1 << (46-16);            // Habilita interrupcao PORTA
	NVIC_ICPR |= 1 << (46-16);
	NVIC_IPR7 |= NVIC_IP_PRI_30(priority << 6);    // Nivel de prioridade = 3

	// Comfigurar o trigger
	PORTA_PCR4 |= PORT_PCR_ISF_MASK | 
			PORT_PCR_IRQC(0xA);           // "Interrupt on falling edge"
	PORTA_PCR5 |= PORT_PCR_ISF_MASK | 
			PORT_PCR_IRQC(0xA);           // "Interrupt on falling edge"
	PORTA_PCR12 |= PORT_PCR_ISF_MASK | 
			PORT_PCR_IRQC(0xA);            // "Interrupt on falling edge"
}

/**
 * @brief Rotina de servio para PORTA (Vetor 46, IRQ30, prioridade em NVICIPR7[31:30])
 */
void PORTA_IRQHandler(void) {
	rgb_bool_type cor[3]={{HABILITA,DESABILITA,DESABILITA},
			{DESABILITA,HABILITA,DESABILITA},
			{DESABILITA,DESABILITA,HABILITA}};

	if (PORTA_PCR4 & PORT_PCR_ISF_MASK) {
		// liga led vermelho e apaga outros
		GPIO_ledRGB (cor[VERMELHO]);
		// limpa flag
		PORTA_PCR4 |= PORT_PCR_ISF_MASK; 
		return;
	}
	
	if (PORTA_PCR5 & PORT_PCR_ISF_MASK) {
		// liga led verde e apaga outros
		GPIO_ledRGB (cor[VERDE]);
		// limpa flag
		PORTA_PCR5 |= PORT_PCR_ISF_MASK;
		return;
	} 
	
	if (PORTA_PCR12 & PORT_PCR_ISF_MASK) {
		// liga led azul e apaga outros
		GPIO_ledRGB (cor[AZUL]);
		// limpa flag
		PORTA_PCR12 |= PORT_PCR_ISF_MASK;
		
		return;
	}
}

/**
 * @brief controle o estado do led RGB pelas botoeiras.
 * @return 1 somente para satisfazer a sintaxe C
 */
int main (void)
{		
	GPIO_initLedRGB();
	GPIO_initSwitches();
	GPIO_enableSwInterrupt(3);
	
	for(;;) 
	{
	}
	return 1;
}

